/*
 * Copyright (c) 1993 The Department of Computer Science, University of Calgary.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      Calgary, Department of Computer Science and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <sys/ioctl.h>

/* these pacify cpp */
#include <sys/time.h>
#include <sys/procfs.h>
#include <sys/types.h>
#include <net/route.h>
#include <net/if.h>

static
char sccsid[] =
    "@(#)ioctl.c 1.3 93/12/01 jda";

pr_ioctl(fp, flags)
FILE *fp;
{
	int f = flags & 0xffff;

	switch (f & IOCTYPE) {
	    case LIOC:
		switch (f) {
		    case LIOCGETP:	return(fprintf(fp, "LIOCGETP"));
		    case LIOCSETP:	return(fprintf(fp, "LIOCSETP"));
		    case LIOCGETS:	return(fprintf(fp, "LIOCGETS"));
		    case LIOCSETS:	return(fprintf(fp, "LIOCSETS"));
		}
		break;

	    case DIOC:
		switch (f) {
		    case DIOCGETC:	return(fprintf(fp, "DIOCGETC"));
		    case DIOCGETB:	return(fprintf(fp, "DIOCGETB"));
		    case DIOCSETE:	return(fprintf(fp, "DIOCSETE"));
		}
		break;

	    case TIOC:
		switch (f) {
		    case TCSBRK:	return(fprintf(fp, "TCSBRK"));
		    case TCXONC:	return(fprintf(fp, "TCXONC"));
		    case TCFLSH:	return(fprintf(fp, "TCFLSH"));
		    case TCGETA:	return(fprintf(fp, "TCGETA"));
		    case TCSETA:	return(fprintf(fp, "TCSETA"));
		    case TCSETAW:	return(fprintf(fp, "TCSETAW"));
		    case TCSETAF:	return(fprintf(fp, "TCSETAF"));
		    case TIOCFLUSH:	return(fprintf(fp, "TIOCFLUSH"));
		    case TCDSET:	return(fprintf(fp, "TCDSET"));
		    case TCBLKMD:	return(fprintf(fp, "TCBLKMD"));
		    case TIOCPKT:	return(fprintf(fp, "TIOCPKT"));
		    case TIOCNOTTY:	return(fprintf(fp, "TIOCNOTTY"));
		    case TIOCSTI:	return(fprintf(fp, "TIOCSTI"));
		}
		break;


	    case tIOC:
		switch (flags) {
		    case TIOCSPGRP:	return(fprintf(fp, "TIOCSPGRP"));
		    case TIOCGPGRP:	return(fprintf(fp, "TIOCGPGRP"));
		    case TIOCCONS:	return(fprintf(fp, "TIOCCONS"));
		    case TIOCGWINSZ:	return(fprintf(fp, "TIOCGWINSZ"));
		    case TIOCSWINSZ:	return(fprintf(fp, "TIOCSWINSZ"));
		}
		switch (f) {
		    case TIOCGETD:		return(fprintf(fp, "TIOCGETD"));
		    case TIOCSETD:		return(fprintf(fp, "TIOCSETD"));
		    case TIOCHPCL:		return(fprintf(fp, "TIOCHPCL"));
		    case TIOCGETP:		return(fprintf(fp, "TIOCGETP"));
		    case TIOCSETP:		return(fprintf(fp, "TIOCSETP"));
		    case TIOCSETN:		return(fprintf(fp, "TIOCSETN"));
		    case TIOCEXCL:		return(fprintf(fp, "TIOCEXCL"));
		    case TIOCNXCL:		return(fprintf(fp, "TIOCNXCL"));
		    case TIOCSETC:		return(fprintf(fp, "TIOCSETC"));
		    case TIOCGETC:		return(fprintf(fp, "TIOCGETC"));
		    case TIOCLBIS:		return(fprintf(fp, "TIOCLBIS"));
		    case TIOCLBIC:		return(fprintf(fp, "TIOCLBIC"));
		    case TIOCLSET:		return(fprintf(fp, "TIOCLSET"));
		    case TIOCLGET:		return(fprintf(fp, "TIOCLGET"));
		    case TIOCSBRK:		return(fprintf(fp, "TIOCSBRK"));
		    case TIOCCBRK:		return(fprintf(fp, "TIOCCBRK"));
		    case TIOCSDTR:		return(fprintf(fp, "TIOCSDTR"));
		    case TIOCCDTR:		return(fprintf(fp, "TIOCCDTR"));
		    case TIOCSLTC:		return(fprintf(fp, "TIOCSLTC"));
		    case TIOCGLTC:		return(fprintf(fp, "TIOCGLTC"));
		    case TIOCOUTQ:		return(fprintf(fp, "TIOCOUTQ"));
		    case TIOCSTOP:		return(fprintf(fp, "TIOCSTOP"));
		    case TIOCSTART:		return(fprintf(fp, "TIOCSTART"));
		    case TIOCGSID:		return(fprintf(fp, "TIOCGSID"));
		    case TIOCSSID:		return(fprintf(fp, "TIOCSSID"));
		    case TIOCMSET:		return(fprintf(fp, "TIOCMSET"));
		    case TIOCMBIS:		return(fprintf(fp, "TIOCMBIS"));
		    case TIOCMBIC:		return(fprintf(fp, "TIOCMBIC"));
		    case TIOCMGET:		return(fprintf(fp, "TIOCMGET"));
		    case TIOCREMOTE:		return(fprintf(fp, "TIOCREMOTE"));
		    case TIOCSIGNAL:		return(fprintf(fp, "TIOCSIGNAL"));
		}
		break;

	    case TFIOC:
		switch (f) {
		    case oFIONREAD:	return(fprintf(fp, "oFIONREAD"));
		}
		break;

	    case LDIOC:
		switch (f) {
		    case LDOPEN:	return(fprintf(fp, "LDOPEN"));
		    case LDCLOSE:	return(fprintf(fp, "LDCLOSE"));
		    case LDCHG:		return(fprintf(fp, "LDCHG"));
		    case LDGETT:	return(fprintf(fp, "LDGETT"));
		    case LDSETT:	return(fprintf(fp, "LDSETT"));
		}
		break;

	    case PIOC:
		switch (f) {
		    case PIOCSTATUS:	return(fprintf(fp, "PIOCSTATUS"));
		    case PIOCSTOP:	return(fprintf(fp, "PIOCSTOP"));
		    case PIOCWSTOP:	return(fprintf(fp, "PIOCWSTOP"));
		    case PIOCRUN:	return(fprintf(fp, "PIOCRUN"));
		    case PIOCGTRACE:	return(fprintf(fp, "PIOCGTRACE"));
		    case PIOCSTRACE:	return(fprintf(fp, "PIOCSTRACE"));
		    case PIOCSSIG:	return(fprintf(fp, "PIOCSSIG"));
		    case PIOCKILL:	return(fprintf(fp, "PIOCKILL"));
		    case PIOCUNKILL:	return(fprintf(fp, "PIOCUNKILL"));
		    case PIOCGHOLD:	return(fprintf(fp, "PIOCGHOLD"));
		    case PIOCSHOLD:	return(fprintf(fp, "PIOCSHOLD"));
		    case PIOCMAXSIG:	return(fprintf(fp, "PIOCMAXSIG"));
		    case PIOCACTION:	return(fprintf(fp, "PIOCACTION"));
		    case PIOCGFAULT:	return(fprintf(fp, "PIOCGFAULT"));
		    case PIOCSFAULT:	return(fprintf(fp, "PIOCSFAULT"));
		    case PIOCCFAULT:	return(fprintf(fp, "PIOCCFAULT"));
		    case PIOCGENTRY:	return(fprintf(fp, "PIOCGENTRY"));
		    case PIOCSENTRY:	return(fprintf(fp, "PIOCSENTRY"));
		    case PIOCGEXIT:	return(fprintf(fp, "PIOCGEXIT"));
		    case PIOCSEXIT:	return(fprintf(fp, "PIOCSEXIT"));
		    case PIOCSFORK:	return(fprintf(fp, "PIOCSFORK"));
		    case PIOCRFORK:	return(fprintf(fp, "PIOCRFORK"));
		    case PIOCSRLC:	return(fprintf(fp, "PIOCSRLC"));
		    case PIOCRRLC:	return(fprintf(fp, "PIOCRRLC"));
		    case PIOCGREG:	return(fprintf(fp, "PIOCGREG"));
		    case PIOCSREG:	return(fprintf(fp, "PIOCSREG"));
		    case PIOCGFPREG:	return(fprintf(fp, "PIOCGFPREG"));
		    case PIOCSFPREG:	return(fprintf(fp, "PIOCSFPREG"));
		    case PIOCNICE:	return(fprintf(fp, "PIOCNICE"));
		    case PIOCPSINFO:	return(fprintf(fp, "PIOCPSINFO"));
		    case PIOCNMAP:	return(fprintf(fp, "PIOCNMAP"));
		    case PIOCMAP:	return(fprintf(fp, "PIOCMAP"));
		    case PIOCOPENM:	return(fprintf(fp, "PIOCOPENM"));
		    case PIOCCRED:	return(fprintf(fp, "PIOCCRED"));
		    case PIOCGROUPS:	return(fprintf(fp, "PIOCGROUPS"));
		    case PIOCGETPR:	return(fprintf(fp, "PIOCGETPR"));
		    case PIOCGETU:	return(fprintf(fp, "PIOCGETU"));
		    case PIOCSET:	return(fprintf(fp, "PIOCSET"));
		    case PIOCRESET:	return(fprintf(fp, "PIOCRESET"));
		    case PIOCNWATCH:	return(fprintf(fp, "PIOCNWATCH"));
		    case PIOCGWATCH:	return(fprintf(fp, "PIOCGWATCH"));
		    case PIOCSWATCH:	return(fprintf(fp, "PIOCSWATCH"));
		    case PIOCUSAGE:	return(fprintf(fp, "PIOCUSAGE"));
		}
		break;

	    case ('f'<<8):
		switch (flags) {
		    case FIONREAD:	return(fprintf(fp, "FIONREAD"));
		    case FIONBIO:	return(fprintf(fp, "FIONBIO"));
		    case FIOASYNC:	return(fprintf(fp, "FIOASYNC"));
		    case FIOSETOWN:	return(fprintf(fp, "FIOSETOWN"));
		    case FIOGETOWN:	return(fprintf(fp, "FIOGETOWN"));
		}
		break;

#ifdef IRIX5
	    case ('s'<<8):
		switch (flags) {
		    case SIOCSHIWAT:	return(fprintf(fp, "SIOCSHIWAT"));
		    case SIOCGHIWAT:	return(fprintf(fp, "SIOCGHIWAT"));
		    case SIOCSLOWAT:	return(fprintf(fp, "SIOCSLOWAT"));
		    case SIOCGLOWAT:	return(fprintf(fp, "SIOCGLOWAT"));
		    case SIOCATMARK:	return(fprintf(fp, "SIOCATMARK"));
		    case SIOCSPGRP:	return(fprintf(fp, "SIOCSPGRP"));
		    case SIOCGPGRP:	return(fprintf(fp, "SIOCGPGRP"));
		    case SIOCNREAD:	return(fprintf(fp, "SIOCNREAD"));
		}
		break;
#endif

#ifdef IRIX5
	    case ('r'<<8):
		switch (flags) {
		    case SIOCADDRT:	return(fprintf(fp, "SIOCADDRT"));
		    case SIOCDELRT:	return(fprintf(fp, "SIOCDELRT"));
		    case SIOCSETRTINFO:	return(fprintf(fp, "SIOCSETRTINFO"));
		    case SIOCGETRTINFO:	return(fprintf(fp, "SIOCGETRTINFO"));
		}
		break;
#endif

#ifdef IRIX5
	    case ('i'<<8):
		switch (flags) {
		    case SIOCSIFADDR:	return(fprintf(fp, "SIOCSIFADDR"));
		    case SIOCGIFADDR:	return(fprintf(fp, "SIOCGIFADDR"));
		    case SIOCSIFDSTADDR:return(fprintf(fp, "SIOCSIFDSTADDR"));
		    case SIOCGIFDSTADDR:return(fprintf(fp, "SIOCGIFDSTADDR"));
		    case SIOCSIFFLAGS:	return(fprintf(fp, "SIOCSIFFLAGS"));
		    case SIOCGIFFLAGS:	return(fprintf(fp, "SIOCGIFFLAGS"));
		    case SIOCGIFBRDADDR:return(fprintf(fp, "SIOCGIFBRDADDR"));
		    case SIOCSIFBRDADDR:return(fprintf(fp, "SIOCSIFBRDADDR"));
		    case SIOCGIFCONF:	return(fprintf(fp, "SIOCGIFCONF"));
		    case SIOCGIFNETMASK:return(fprintf(fp, "SIOCGIFNETMASK"));
		    case SIOCSIFNETMASK:return(fprintf(fp, "SIOCSIFNETMASK"));
		    case SIOCGIFMETRIC:	return(fprintf(fp, "SIOCGIFMETRIC"));
		    case SIOCSIFMETRIC:	return(fprintf(fp, "SIOCSIFMETRIC"));
		    case SIOCSARP:	return(fprintf(fp, "SIOCSARP"));
		    case SIOCGARP:	return(fprintf(fp, "SIOCGARP"));
		    case SIOCDARP:	return(fprintf(fp, "SIOCDARP"));
		    case SIOCADDMULTI:	return(fprintf(fp, "SIOCADDMULTI"));
		    case SIOCDELMULTI:	return(fprintf(fp, "SIOCDELMULTI"));
		    case SIOCSIFSTATS:	return(fprintf(fp, "SIOCSIFSTATS"));
		    case SIOCGIFSTATS:	return(fprintf(fp, "SIOCGIFSTATS"));
		    case SIOCSIFHEAD:	return(fprintf(fp, "SIOCSIFHEAD"));
		}
		break;
#endif
	}

	fprintf(fp, "0x%x", flags);
}
