/*
 * Copyright (c) 1993 The Department of Computer Science, University of Calgary.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      Calgary, Department of Computer Science and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#define _KERNEL
#include <sys/ksignal.h>
#include <string.h>

/*
 *	The signal(2) "system" call appears to be more like BSD's
 *	sigvec().. it has three parameters, not just two, which
 *	resemble a signal number and two pointers.  The signal
 *	number has some bits ORed in which are defined under an
 *	#ifdef _KERNEL in <sys/signal.h>.
 */

static
char sccsid[] =
    "@(#)signal.c 1.4 93/12/01 jda";

char *sigmap(n)
{
	static char buf[32];
	static char *map[] = {
		"SIGHUP",
		"SIGINT",
		"SIGQUIT",
		"SIGILL",
		"SIGTRAP",
		"SIGABRT",
		"SIGEMT",
		"SIGFPE",
		"SIGKILL",
		"SIGBUS",
		"SIGSEGV",
		"SIGSYS",
		"SIGPIPE",
		"SIGALRM",
		"SIGTERM",
		"SIGUSR1",
		"SIGUSR2",
		"SIGCLD",
		"SIGPWR",
		"SIGSTOP",
		"SIGTSTP",
		"SIGPOLL",
		"SIGIO",
		"SIGURG",
		"SIGWINCH",
		"SIGVTALRM",
		"SIGPROF",
		"SIGCONT",
		"SIGTTIN",
		"SIGTTOU",
		"SIGXCPU",
		"SIGXFSZ",
	};

	*buf = '\0';
	switch (n &~ SIGNO_MASK) {
	    case 0:						break;
	    case SIGDEFER:	strcpy(buf, "SIGDEFER|");	break;
	    case SIGHOLD:	strcpy(buf, "SIGHOLD|");	break;
	    case SIGRELSE:	strcpy(buf, "SIGRELSE|");	break;
	    case SIGIGNORE:	strcpy(buf, "SIGIGNORE|");	break;
	    case SIGPAUSE:	strcpy(buf, "SIGPAUSE|");	break;
	    default:
		sprintf(buf, "0x%x", n);
		return(buf);
	}
	n &= SIGNO_MASK;

	if ((unsigned int)--n >= sizeof(map)/sizeof(char *))
		die("signal number %d out of range", ++n);
	return(strcat(buf, map[n]));
}
