/*
 * Copyright (c) 1993 The Department of Computer Science, University of Calgary.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      Calgary, Department of Computer Science and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <mntent.h>
#include <varargs.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/procfs.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <signal.h>
#include <unistd.h>

#define USAGE	"Usage: %s [-st] [-lstrlen] [-ofile] -ppid|command\n"

static
char sccsid[] =
    "@(#)trace.c 1.10 93/12/01 jda";

int in_syscall, last_callno;
int dash_s, dash_t, dash_l = 32;
char *progname, dir[MAXPATHLEN+1], *dash_p;
FILE *ofp = stderr;

extern char *sigmap();

die(va_alist)
va_dcl
{
	char *fmt;
	va_list ap;

	va_start(ap);
	  fmt = va_arg(ap, char *);
	  fprintf(stderr, "%s: ", progname);
	  vfprintf(stderr, fmt, ap);
	va_end(ap);
	fputc('\n', stderr);
	exit(!0);
}

finddbgfs()
{
	FILE *fp;
	struct mntent *mp;

	fp = setmntent(MOUNTED, "r");
	if (!fp)
		die("couldn't read `%s'", MOUNTED);

	for (mp = getmntent(fp); mp; mp = getmntent(fp))
		if (!strcmp(mp->mnt_type, MNTTYPE_DBG))
			break;
	if (mp) {
		sprintf(dir, "%s/", mp->mnt_dir);
		endmntent(fp);
		return;
	}
	die("couldn't find %s fs", MNTTYPE_DBG);
}

#define EXIT	2

Ioctl(a, b, c, d)
char *d;
{
	int status, pid, oldpid;

	if (ioctl(a, b, c) < 0) {
		if (errno == ENOENT) {
			if (dash_p) {
				fprintf(stderr,
					"\nprocess %s terminated\n",
					dash_p);
				exit(0);
			}

			while ((oldpid = wait(&status)) >= 0)
				pid = oldpid;
			if (!WIFEXITED(status))
				/* can this happen? */
				fprintf(stderr,
					"\nprocess %d terminated\n",
					pid);
			else if (!in_syscall || last_callno != EXIT)
				fprintf(stderr,
					"\nprocess %d exited with %d\n",
					pid, WEXITSTATUS(status));
			else
				fprintf(ofp, "%d) = ?\n", WEXITSTATUS(status));
			exit(0);
				
		}
		die("%s (%s)", d, strerror(errno));
	}
}

/* setup for tracing already-running process */

p_int_h()
{
	fprintf(stderr, "\nprocess %s detached\n", dash_p);
	exit(0);
}

p_setup(fd)
{
	sigset_t sigt;
	sysset_t syst;

	prfillset(&sigt);
	prfillset(&syst);

	Ioctl(fd, PIOCSTRACE, &sigt, "PIOCSTRACE");
	Ioctl(fd, PIOCSENTRY, &syst, "PIOCSENTRY");
	Ioctl(fd, PIOCSEXIT, &syst, "PIOCSEXIT");

	/* for the time being, don't care about children */
	{
		long flags;

		flags = PR_RLC;
		Ioctl(fd, PIOCSET, &flags, "PIOCSET");
		flags = PR_FORK|PR_KLC;
		Ioctl(fd, PIOCRESET, &flags, "PIOCRESET");
	}

	if (!dash_s)
		Ioctl(fd, PIOCSTOP, NULL, "PIOCSTOP");

	fprintf(stderr, "process %s attached - interrupt to quit\n", dash_p);
	signal(SIGINT, p_int_h);
}

char *getpath(file)
char *file;
{
	int found = 0;
	char *PATH, *p;
	struct stat st;
	static char buf[MAXPATHLEN];

	if (strchr(file, '/')) {
		strcpy(buf, file);
		if (!access(buf, X_OK) && stat(buf, &st) >= 0) {
			if (S_ISREG(st.st_mode))
				goto Y;
			found = !0;
		}
		goto X;
	}

	PATH = ((p = (char *)getenv("PATH")) ? p : "");

	while (1) {
		*buf = '\0';
		p = ((p = strchr(PATH, ':')) ? p : strchr(PATH, '\0'));

		if (p-PATH > 1) {
			strncpy(buf, PATH, p-PATH);
			buf[p-PATH] = '/';
			buf[p-PATH+1] = '\0';
		}
		strcat(buf, file);

		if (!access(buf, X_OK)) {
			if (stat(buf, &st) >= 0) {
				if (S_ISREG(st.st_mode))
					goto Y;
				found = !0;
			}
		}

		if (!*p)
X:			die("%s", strerror(found ? EACCES : ENOENT));
		PATH = p+1;
	}

Y:	if (access(buf, R_OK))
		die("need read permission to trace");
	return(buf);
}

#define EXECVE	60

c_null_h() {}

c_setup(i, argv)
char **argv;
{
	int fd, pid;
	char *cmd = getpath(argv[i]);
	sigset_t sigt;
	sysset_t syst;

	switch (pid = fork()) {
	    case -1:
		die("fork() failed");
	    case 0:
		signal(SIGCONT, c_null_h);
		pause();				/* synchronize */
		execv(cmd, &argv[i]);
		die("couldn't exec %s", cmd);
	}

	sprintf(dir + strlen(dir), "%d", pid);
	fd = open(dir, O_RDWR);
	if (fd < 0) {
		if (errno != ENOENT)
			die(strerror(errno));
		else {
			while (wait(NULL) >= 0)
				;
			exit(!0);
		}
	}

	premptyset(&syst);
	praddset(&syst, EXECVE);
	Ioctl(fd, PIOCSEXIT, &syst, "PIOCSEXIT");

	/* for the time being, don't care about children */
	{
		long flags;

		flags = PR_KLC;
		Ioctl(fd, PIOCSET, &flags, "PIOCSET");
		flags = PR_FORK|PR_RLC;
		Ioctl(fd, PIOCRESET, &flags, "PIOCRESET");
	}

	/*
	 *  Start up child again, who should be waiting for
	 *  us to send a SIGCONT.  We then wait for the child's
	 *  execve() before starting the trace.  If any set[ug]id
	 *  bits are set on the exec'ed file, then we must re-open
	 *  the proc file.
	 */
	kill(pid, SIGCONT);
	if (ioctl(fd, PIOCWSTOP, NULL) < 0) {
		int newfd;
		long flags;
		prstatus_t prs;

		if (errno != EAGAIN)
			die("PIOCWSTOP (%s)", strerror(errno));

		newfd = open(dir, O_RDWR);
		if (newfd < 0) {
			if (errno != ENOENT)
				die(strerror(errno));
			else {
				while (wait(NULL) >= 0)
					;
				exit(!0);
			}
		}
		close(fd);
		fd = newfd;

		/* RLC is set on an exec err */
		flags = PR_KLC;
		Ioctl(fd, PIOCSET, &flags, "PIOCSET");
		flags = PR_FORK|PR_RLC;
		Ioctl(fd, PIOCRESET, &flags, "PIOCRESET");
	}

	prfillset(&sigt);
	prfillset(&syst);
	Ioctl(fd, PIOCSTRACE, &sigt, "PIOCSTRACE");
	Ioctl(fd, PIOCSENTRY, &syst, "PIOCSENTRY");
	Ioctl(fd, PIOCSEXIT, &syst, "PIOCSEXIT");
	
	Ioctl(fd, PIOCRUN, NULL, "PIOCRUN");
	return(fd);
}

trace(fd)
{
	int rv;
	prstatus_t prs;
	fd_set fds, OLDfds;
	struct timeval tv;

	in_syscall = 0;
	FD_ZERO(&OLDfds);
	FD_SET(fd, &OLDfds);

	while (1) {
		fds = OLDfds;
		if (in_syscall) {
			tv.tv_sec = 0;
			tv.tv_usec = 100000;
		}

		rv = select(fd+1, NULL, NULL, &fds, (in_syscall ? &tv : NULL));
		switch (rv) {
		    case -1:
			die("select() failed");
		    case 0:
			Ioctl(fd, PIOCSTATUS, &prs, "PIOCSTATUS");
			if (prs.pr_flags & PR_ASLEEP) {
				pr_syscall(fd, ofp, &prs);
				in_syscall = 0;
			}
			continue;
		    case 1:
			Ioctl(fd, PIOCWSTOP, &prs, "PIOCWSTOP");
			break;
		}

		switch (prs.pr_why) {
		    case PR_REQUESTED:
			if (prs.pr_syscall) {
				last_callno = prs.pr_syscall;
				pr_callname(ofp, &prs);
				pr_syscall(fd, ofp, &prs);
			}
			break;
		    case PR_SIGNALLED:
			fprintf(ofp,
				"[%s]\n",
				sigmap(prs.pr_what));
			break;
		    case PR_SYSENTRY:
			in_syscall = !0;
			last_callno = prs.pr_syscall;
			pr_callname(ofp, &prs);
			break;
		    case PR_SYSEXIT:
			if (in_syscall) {
				pr_syscall(fd, ofp, &prs);
				in_syscall = 0;
			}
			pr_sysret(ofp, &prs);
			break;
		    default:
			die("no idea why this stopped: %d", prs.pr_why);
		}

		Ioctl(fd, PIOCRUN, NULL, "PIOCRUN");
	}
}

main(argc, argv)
char **argv;
{
	int ch, fd;
	char *dash_o = NULL, proc[64];
	extern int optind, opterr;
	extern char *optarg;

	(!(progname = strrchr(argv[0],'/'))) ? progname = argv[0] : progname++;
	opterr = 0;

	while ((ch = getopt(argc, argv, "stl:o:p:")) >= 0) {
		switch (ch) {
		    case 's':
			dash_s = !0;
			break;
		    case 't':
			dash_t = !0;
			break;
		    case 'l':
			dash_l = atoi(optarg);
			break;
		    case 'o':
			dash_o = optarg;
			break;
		    case 'p':
			/* idiot guard */
			sprintf(proc, "%d", strtol(optarg, NULL, 0));
			dash_p = proc;
			break;
		    default:
			fprintf(stderr, USAGE, progname);
			exit(!0);
		}
	}

	if (dash_o) {
		ofp = fopen(dash_o, "w");
		if (!ofp)
			die("couldn't open `%s' for writing", dash_o);
	}

	finddbgfs();

	if (optind == argc) {
		if (!dash_p)
			die("no command to trace");

		strcat(dir, dash_p);
		fd = open(dir, O_RDWR);
		if (fd < 0)
			die(errno == ENOENT ? "no such process"
					    : strerror(errno));
		p_setup(fd);
		trace(fd);
	} else {
		if (dash_p)
			die("can't have -p and a command");
		if (dash_s)
			die("-s meaningless unless -p used");

		fd = c_setup(optind, argv);
		trace(fd);
	}

	exit(0);
}
